@@ -44,7 +44,7 @@
 
 #define GTR_TAB_SAVE_AS "gtr-tab-save-as"
 
-static void load_file_list (GtrWindow * window, const GSList * uris);
+static void load_file_list (GtrWindow * window, GSList * uris);
 static GList * get_modified_documents (GtrWindow * window);
 
 typedef struct {
@@ -61,88 +61,6 @@ user_data_free (UserData *ud)
   g_free (ud);
 }
 
-/*
- * The main file opening function. Checks that the file isn't already open,
- * and if not, opens it in a new tab.
- */
-gboolean
-gtr_open (GFile * location, GtrWindow * window, GError ** error)
-{
-  GtrPo *po;
-  GtrTab *tab;
-  GList *current;
-  GtrView *active_view;
-  GtrHeader *header;
-  gchar *dl_lang;
-  gchar *dl_module;
-  gchar *dl_branch;
-  gchar *dl_domain;
-  gchar *dl_module_state;
-
-  /*
-   * If the filename can't be opened, pass the error back to the caller
-   * to handle.
-   */
-
-  po = gtr_po_new ();
-  if (!gtr_po_parse (po, location, error)) {
-    gtr_window_show_projects (window);
-    // TODO: show error message
-    return FALSE;
-  }
-
-  if ((*error != NULL)
-      && (((GError *) * error)->code != GTR_PO_ERROR_RECOVERY)) {
-    gtr_window_show_projects (window);
-    return FALSE;
-  }
-
-  header = gtr_po_get_header (po);
-  dl_lang = gtr_header_get_dl_lang (header);
-  dl_module = gtr_header_get_dl_module (header);
-  dl_branch = gtr_header_get_dl_branch (header);
-  dl_domain = gtr_header_get_dl_domain (header);
-  dl_module_state = gtr_header_get_dl_state (header);
-
-  /*
-   * Set Damned Lies info when a po file is opened locally
-   */
-  gtr_po_set_dl_info(po,
-                     dl_lang,
-                     dl_module,
-                     dl_branch,
-                     dl_domain,
-                     dl_module_state);
-
-  /*
-   * Create a page to add to our list of open files
-   */
-  tab = gtr_window_create_tab (window, po);
-
-  /*
-   * Activate the upload file icon if the po file is in the appropriate
-   * state as on the vertimus workflow
-   */
-  //active_notebook = gtr_window_get_notebook (window);
-  gtr_tab_enable_upload (tab, gtr_po_can_dl_upload (po));
-
-  /*
-   * Show the current message.
-   */
-  current = gtr_po_get_current_message (po);
-  gtr_tab_message_go_to (tab, current->data, FALSE, GTR_TAB_MOVE_NONE);
-
-  /*
-   * Grab the focus
-   */
-  active_view = gtr_tab_get_active_view (tab);
-  gtk_widget_grab_focus (GTK_WIDGET (active_view));
-
-  gtr_window_show_poeditor (window);
-
-  return TRUE;
-}
-
 static void
 gtr_po_parse_files_from_dialog (GObject *source, GAsyncResult *res, void *user_data)
 {
@@ -165,7 +83,7 @@ gtr_po_parse_files_from_dialog (GObject *source, GAsyncResult *res, void *user_d
   /*
    * Open the file via our centralized opening function.
    */
-  locations = g_slist_append (locations, file);
+  locations = g_slist_append (locations, g_steal_pointer (&file));
   load_file_list (window, locations);
 }
 
@@ -229,7 +147,7 @@ gtr_want_to_save_current_dialog (GtrWindow * window, void (*callback)(GtrWindow
 static void
 gtr_open_file_dialog_nocheck (GtrWindow *window)
 {
-  GtkFileDialog *dialog;
+  g_autoptr(GtkFileDialog) dialog = NULL;
 
   dialog = gtr_file_chooser_new (GTK_WINDOW (window),
                                  FILESEL_OPEN,
@@ -382,6 +300,7 @@ gtr_upload_file (GtkWidget *upload_dialog,
   GtrProfile *profile;
   GtrHeader *header;
   g_autoptr (SoupMultipart) mpart = NULL;
+  g_autoptr (GFile) location_file = NULL;
 
   SoupMessage *msg = NULL;
   static SoupSession *session = NULL;
@@ -405,8 +324,9 @@ gtr_upload_file (GtkWidget *upload_dialog,
   /* Get file content */
   tab = gtr_window_get_active_tab (window);
   po = gtr_tab_get_po (tab);
-  filename = g_file_get_basename (gtr_po_get_location (po));
-  g_file_load_contents (gtr_po_get_location (po), NULL, &content, &size, NULL,
+  location_file = gtr_po_get_location (po);
+  filename = g_file_get_basename (location_file);
+  g_file_load_contents (location_file, NULL, &content, &size, NULL,
                         &error);
   if (error != NULL)
     g_warning ("Error opening file %s: %s", filename, (error)->message);
@@ -496,7 +416,7 @@ gtr_upload_file_dialog (GtrWindow * window)
 void
 gtr_save_file_as_dialog (GtrWindow * window)
 {
-  GtkFileDialog *dialog;
+  g_autoptr(GtkFileDialog) dialog = NULL;
 
   dialog = gtr_file_chooser_new (GTK_WINDOW (window),
                                  FILESEL_SAVE,
@@ -506,23 +426,34 @@ gtr_save_file_as_dialog (GtrWindow * window)
 }
 
 static void
-load_file_list (GtrWindow * window, const GSList * locations)
+load_file_list (GtrWindow * window, GSList * locations)
 {
   GSList *locations_to_load = NULL;
   g_autoptr (GError) error = NULL;
 
   g_return_if_fail ((locations != NULL) && (locations->data != NULL));
 
-  gtr_window_remove_tab (window);
-
   locations_to_load = g_slist_reverse ((GSList*) locations);
 
   while (locations_to_load != NULL)
     {
+      g_autoptr (GtrPo) po = NULL;
+
       g_return_if_fail (locations_to_load->data != NULL);
 
-      if (!gtr_open (locations_to_load->data, window, &error))
-        break;
+      po = gtr_po_new_from_file (locations_to_load->data, &error);
+      if (error)
+        {
+          g_log (NULL,
+                 G_LOG_LEVEL_WARNING,
+                 "Could not load po file %s",
+                 g_file_get_path (locations_to_load->data));
+          break;
+        }
+
+      if (gtr_window_get_active_tab (window) != NULL)
+        window = gtr_application_create_window (GTR_APP);
+      gtr_window_set_po (window, po);
 
       locations_to_load = g_slist_next (locations_to_load);
     }
@@ -539,7 +470,7 @@ load_file_list (GtrWindow * window, const GSList * locations)
       adw_dialog_present (dialog, GTK_WIDGET (window));
     }
 
-  g_slist_free ((GSList *) locations_to_load);
+  g_slist_free_full (locations, g_object_unref);
 }
 
 
@@ -549,7 +480,7 @@ load_file_list (GtrWindow * window, const GSList * locations)
  * Ignore non-existing URIs
  */
 void
-gtr_actions_load_locations (GtrWindow * window, const GSList * locations)
+gtr_actions_load_locations (GtrWindow * window, GSList * locations)
 {
   g_return_if_fail (GTR_IS_WINDOW (window));
   g_return_if_fail ((locations != NULL) && (locations->data != NULL));
@@ -593,15 +524,15 @@ close_confirmation_dialog_response_handler (GtrCloseConfirmationDialog *dlg,
                                             GtrWindow *window)
 {
   GList *selected_documents;
+
+  adw_dialog_close (ADW_DIALOG (dlg));
   if (g_strcmp0 (response, "yes") == 0)
     {
       /* Save and Close */
       selected_documents = gtr_close_confirmation_dialog_get_selected_documents (dlg);
       if (selected_documents == NULL)
         {
-          adw_dialog_close (ADW_DIALOG (dlg));
           close_all_tabs (window);
-          return;
         }
       else
         {
@@ -611,12 +542,8 @@ close_confirmation_dialog_response_handler (GtrCloseConfirmationDialog *dlg,
     }
   else if (g_strcmp0 (response, "no") == 0)
     {
-      adw_dialog_close (ADW_DIALOG (dlg));
       close_all_tabs (window);
-      return;
     }
-
-  adw_dialog_close (ADW_DIALOG (dlg));
 }
 
 void
@@ -709,11 +636,6 @@ close_all_documents (GtrWindow * window, gboolean logout_mode)
   else
     {
       close_all_tabs (window);
-
-      if (logout_mode)
-        {
-          gtk_window_destroy (GTK_WINDOW (window));
-        }
     }
 }
 
