@@ -52,8 +52,6 @@ void gtr_save_file_as_dialog (GtrWindow * window);
 
 void gtr_upload_file_dialog (GtrWindow * window);
 
-gboolean gtr_open (GFile * location, GtrWindow * window, GError ** error);
-
 void gtr_close_tab (GtrTab * tab, GtrWindow * window);
 
 void gtr_file_close (GtrWindow * window);
@@ -61,7 +59,7 @@ void gtr_file_close (GtrWindow * window);
 void gtr_file_quit (GtrWindow * window);
 
 void gtr_actions_load_locations (GtrWindow * window,
-                                 const GSList * locations);
+                                 GSList * locations);
 
 void _gtr_actions_file_close_all (GtrWindow * window);
 
@@ -85,9 +83,6 @@ void gtr_message_status_toggle_fuzzy (GtrWindow * window);
 
 void gtr_actions_edit_clear (GtrWindow * window);
 
-/* View */
-void gtr_actions_view_context (GtrWindow * window);
-
 /*Go*/
 void gtr_message_go_to_first (GtrWindow * window);
 
