@@ -29,7 +29,6 @@
 #include "gtr-actions.h"
 #include "gtr-actions-app.h"
 #include "gtr-application.h"
-#include "gtr-debug.h"
 #include "gtr-dirs.h"
 #include "gtr-settings.h"
 #include "gtr-utils.h"
@@ -108,16 +107,20 @@ ensure_user_config_dir (void)
   return ret;
 }
 
-static gboolean
-on_window_delete_event_cb (GtrWindow * window,
-                           GdkEvent * event, GtrApplication * app)
-{
-  gtr_file_quit (window);
-  return TRUE;
-}
-
 static int
 handle_local_options_cb (GApplication *application, GVariantDict *options, gpointer user_data) {
+  if (g_variant_dict_contains (options, "new-window"))
+    {
+      g_application_register (application, NULL, NULL);
+
+      if (g_application_get_is_remote (application))
+        {
+          g_action_group_activate_action (G_ACTION_GROUP (application),
+                                          "new-window", NULL);
+          return 0;
+        }
+    }
+
   if (g_variant_dict_contains (options, "version")) {
     g_print ("%s - %s\n", PACKAGE, PACKAGE_VERSION);
     return 0;
@@ -135,10 +138,11 @@ gtr_application_init (GtrApplication *application)
   priv->last_dir = NULL;
   priv->first_run = FALSE;
 
-  GOptionEntry options[] = {
-      {"version", 'v', 0, G_OPTION_ARG_NONE, NULL, "Print version information and exit", NULL},
-      {NULL}
-  };
+  GOptionEntry options[] = { { "new-window", 'w', 0, G_OPTION_ARG_NONE, NULL,
+                               "Open a new window", NULL },
+                             { "version", 'v', 0, G_OPTION_ARG_NONE, NULL,
+                               "Print version information and exit", NULL },
+                             { NULL } };
 
   g_application_add_main_option_entries (G_APPLICATION (application), options);
 
@@ -165,6 +169,9 @@ gtr_application_dispose (GObject * object)
   GtrApplication *app = GTR_APPLICATION (object);
   GtrApplicationPrivate *priv = gtr_application_get_instance_private (app);
 
+  g_debug ("Disposing App");
+
+
   g_clear_object (&priv->settings);
   g_clear_object (&priv->window_settings);
 
@@ -179,6 +186,8 @@ gtr_application_finalize (GObject *object)
 
   g_free (priv->last_dir);
 
+  gtk_source_finalize ();
+
   G_OBJECT_CLASS (gtr_application_parent_class)->finalize (object);
 }
 
@@ -517,6 +526,8 @@ gtr_application_startup (GApplication *application)
   g_set_application_name (_("Translation Editor"));
   gtk_window_set_default_icon_name (PACKAGE_APPID);
 
+  g_debug ("App startup");
+
   gtk_source_init ();
 
   g_action_map_add_action_entries (G_ACTION_MAP (application), app_entries,
@@ -581,7 +592,7 @@ gtr_application_setup_window (GApplication *application,
          * malformed files passed on the command-line.
          */
         if (files[i] != NULL)
-          file_list = g_slist_prepend (file_list, files[i]);
+          file_list = g_slist_prepend (file_list, g_object_ref (files[i]));
     }
   window = gtr_application_create_window (GTR_APPLICATION (application));
 
@@ -597,7 +608,6 @@ gtr_application_setup_window (GApplication *application,
     {
       file_list = g_slist_reverse (file_list);
       gtr_actions_load_locations (window, file_list);
-      g_slist_free (file_list);
     }
 }
 
@@ -613,8 +623,17 @@ gtr_application_open (GApplication *application,
 static void
 gtr_application_activate (GApplication *application)
 {
+  GtkWindow *window;
   G_APPLICATION_CLASS (gtr_application_parent_class)->activate (application);
-  gtr_application_setup_window (application, NULL, 0);
+
+  g_debug ("App activate");
+
+  window = gtk_application_get_active_window (GTK_APPLICATION (application));
+
+  if (window != NULL)
+    gtk_window_present (window);
+  else
+    gtr_application_setup_window (application, NULL, 0);
 }
 
 static void
@@ -684,9 +703,6 @@ gtr_application_create_window (GtrApplication *app)
   if (is_maximized)
     gtk_window_maximize (GTK_WINDOW (window));
 
-  g_signal_connect (window, "close-request",
-                    G_CALLBACK (on_window_delete_event_cb), app);
-
   gtk_window_present (GTK_WINDOW (window));
 
   return window;
