@@ -27,9 +27,8 @@ G_BEGIN_DECLS
 
 #define GTR_TYPE_GREETER (gtr_greeter_get_type ())
 
-G_DECLARE_FINAL_TYPE (GtrGreeter, gtr_greeter, GTR, GREETER, AdwBin)
+G_DECLARE_FINAL_TYPE (GtrGreeter, gtr_greeter, GTR, GREETER, AdwNavigationPage)
 
 GtrGreeter* gtr_greeter_new         (GtrWindow *main_window);
-GtkWidget*  gtr_greeter_get_header  (GtrGreeter *self);
 
 G_END_DECLS
