@@ -21,6 +21,7 @@
 
 #include "gtr-message-table-row.h"
 #include "gtr-msg.h"
+#include "gtr-settings.h"
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -40,6 +41,7 @@ typedef struct
   GtkWidget *translated;
 
   GtrMsg *msg;
+  GSettings *ui_settings;
 } GtrMessageTableRowPrivate;
 
 struct _GtrMessageTableRow
@@ -98,7 +100,15 @@ update_msg (GtrMsg             *msg,
 static void
 gtr_message_table_row_init (GtrMessageTableRow *row)
 {
+  GtrMessageTableRowPrivate *priv;
+  priv = gtr_message_table_row_get_instance_private (row);
+  priv->ui_settings = g_settings_new ("org.gnome.gtranslator.preferences.ui");
+
   gtk_widget_init_template (GTK_WIDGET (row));
+
+  g_settings_bind (priv->ui_settings, GTR_SETTINGS_SHOW_ID_COLUMN,
+                   priv->id, "visible",
+                   G_SETTINGS_BIND_DEFAULT);
 }
 
 static void
@@ -109,6 +119,7 @@ gtr_message_table_row_finalize (GObject *object)
 
   priv = gtr_message_table_row_get_instance_private (row);
 
+  g_clear_object (&priv->ui_settings);
   if (priv->msg)
     {
       g_signal_handlers_disconnect_by_func (priv->msg, update_msg, row);
