@@ -27,7 +27,6 @@
 #include <config.h>
 #endif
 
-#include "gtr-debug.h"
 #include "gtr-message-table-row.h"
 #include "gtr-message-table.h"
 #include "gtr-msg.h"
@@ -49,6 +48,7 @@ enum
 typedef struct
 {
   GtkWidget *messages;
+  GtkWidget *idlabel;
   GListStore *store;
   GtkSortListModel *sort_model;
   GtkSingleSelection *selection;
@@ -144,7 +144,6 @@ gtr_message_table_init (GtrMessageTable * table)
   priv->sort_status
       = g_settings_get_enum (priv->ui_settings, GTR_SETTINGS_SORT_ORDER);
   priv->store = NULL;
-  priv->id_sorter = NULL;
 
   gtk_orientable_set_orientation (GTK_ORIENTABLE (table),
                                   GTK_ORIENTATION_VERTICAL);
@@ -155,6 +154,10 @@ gtr_message_table_init (GtrMessageTable * table)
                     G_CALLBACK (on_sort_order_changed), table);
 
   gtk_widget_init_template (GTK_WIDGET (table));
+
+  g_settings_bind (priv->ui_settings, GTR_SETTINGS_SHOW_ID_COLUMN,
+                   priv->idlabel, "visible",
+                   G_SETTINGS_BIND_DEFAULT);
 }
 
 static void
@@ -166,25 +169,12 @@ gtr_message_table_finalize (GObject * object)
   priv = gtr_message_table_get_instance_private (table);
 
   g_clear_object (&priv->ui_settings);
-
-  if (priv->store)
-    {
-      g_object_unref (priv->store);
-      g_object_unref (priv->selection);
-    }
-
-  if (priv->id_sorter)
-    {
-      g_object_unref (priv->id_sorter);
-      g_object_unref (priv->status_sorter);
-      g_object_unref (priv->msg_sorter);
-      g_object_unref (priv->translation_sorter);
-
-      priv->id_sorter = NULL;
-      priv->status_sorter = NULL;
-      priv->msg_sorter = NULL;
-      priv->translation_sorter = NULL;
-    }
+  g_clear_object (&priv->store);
+  g_clear_object (&priv->selection);
+  g_clear_object (&priv->id_sorter);
+  g_clear_object (&priv->status_sorter);
+  g_clear_object (&priv->translation_sorter);
+  g_clear_object (&priv->msg_sorter);
 
   G_OBJECT_CLASS (gtr_message_table_parent_class)->finalize (object);
 }
@@ -263,6 +253,8 @@ gtr_message_table_class_init (GtrMessageTableClass * klass)
                                                "/org/gnome/translator/gtr-message-table.ui");
 
   gtk_widget_class_bind_template_child_private (widget_class, GtrMessageTable, messages);
+
+  gtk_widget_class_bind_template_child_private (widget_class, GtrMessageTable, idlabel);
 }
 
 /**
@@ -329,14 +321,13 @@ gtr_message_table_populate (GtrMessageTable * table, GtrMessageContainer * conta
  * @navigation:
  * @func: (scope call):
  *
- * Returns: (transfer none):
+ * Returns: (transfer full):
  */
 GtrMsg *
 gtr_message_table_navigate (GtrMessageTable * table,
                             GtrMessageTableNavigation navigation,
                             GtrMessageTableNavigationFunc func)
 {
-  GtrMsg *msg = NULL;
   GtrMessageTablePrivate *priv = NULL;
 
   priv = gtr_message_table_get_instance_private (table);
@@ -358,14 +349,15 @@ gtr_message_table_navigate (GtrMessageTable * table,
           int next = gtk_single_selection_get_selected (GTK_SINGLE_SELECTION (priv->selection)) + 1;
           while (TRUE)
             {
-              msg = (GtrMsg*) g_list_model_get_object (G_LIST_MODEL (priv->sort_model), next);
+              g_autoptr(GtrMsg) msg = NULL;
+              msg = GTR_MSG (g_list_model_get_object (G_LIST_MODEL (priv->sort_model), next));
               if (!msg)
                 return NULL;
 
               if (func (msg))
                 {
                   gtk_single_selection_set_selected (GTK_SINGLE_SELECTION (priv->selection), next);
-                  return msg;
+                  return g_steal_pointer (&msg);
                 }
 
               next += 1;
@@ -384,14 +376,15 @@ gtr_message_table_navigate (GtrMessageTable * table,
           int prev = gtk_single_selection_get_selected (GTK_SINGLE_SELECTION (priv->selection)) - 1;
           while (TRUE)
             {
-              msg = (GtrMsg*) g_list_model_get_object (G_LIST_MODEL (priv->sort_model), prev);
+              g_autoptr(GtrMsg) msg = NULL;
+              msg = GTR_MSG (g_list_model_get_object (G_LIST_MODEL (priv->sort_model), prev));
               if (!msg)
                 return NULL;
 
               if (func (msg))
                 {
                   gtk_single_selection_set_selected (GTK_SINGLE_SELECTION (priv->selection), prev);
-                  return msg;
+                  return g_steal_pointer (&msg);
                 }
 
               prev -= 1;
@@ -407,8 +400,9 @@ gtr_message_table_navigate (GtrMessageTable * table,
       break;
     }
 
-  msg = gtk_single_selection_get_selected_item (GTK_SINGLE_SELECTION (priv->selection));
-  return msg;
+  g_autoptr(GtrMsg) msg = NULL;
+  msg = g_object_ref (gtk_single_selection_get_selected_item (GTK_SINGLE_SELECTION (priv->selection)));
+  return g_steal_pointer (&msg);
 }
 
 void
@@ -452,7 +446,6 @@ gtr_message_table_select (GtrMessageTable * table, GtrMsg * msg)
   unsigned int position = 0;
   int i = 0;
   int nitems = 0;
-  GtrMsg *item = NULL;
 
   priv = gtr_message_table_get_instance_private (table);
   nitems = g_list_model_get_n_items (G_LIST_MODEL (priv->selection));
@@ -460,6 +453,7 @@ gtr_message_table_select (GtrMessageTable * table, GtrMsg * msg)
   // look for the msg in the model
   for (i=0; i < nitems; i++)
     {
+      g_autoptr(GtrMsg) item = NULL;
       item = g_list_model_get_item (G_LIST_MODEL (priv->selection), i);
       if (item == msg)
         {
