@@ -32,7 +32,6 @@
 #endif
 
 #include "gtr-application.h"
-#include "gtr-debug.h"
 #include "gtr-file-dialogs.h"
 #include "gtr-po.h"
 #include "gtr-msg.h"
@@ -113,6 +112,8 @@ typedef struct
   /* The state of a DL module */
   gchar *dl_state;
 
+  char *dl_vcs_web;
+
   /* Marks if the file was changed;  */
   guint file_changed : 1;
 } GtrPoPrivate;
@@ -132,22 +133,6 @@ enum
 
 static gchar *message_error = NULL;
 
-static void
-gtr_header_set_field (GtrHeader * header,
-                      const gchar * field, const gchar * data)
-{
-  gchar *msgstr;
-
-  g_return_if_fail (GTR_IS_HEADER (header));
-  g_return_if_fail (data != NULL);
-
-  msgstr = po_header_set_field (gtr_msg_get_msgstr (GTR_MSG (header)),
-                                field, data);
-  gtr_msg_set_msgstr (GTR_MSG (header), msgstr);
-
-  g_free (msgstr);
-}
-
 static void
 gtr_po_set_property (GObject      *object,
                      guint         prop_id,
@@ -227,6 +212,7 @@ gtr_po_init (GtrPo * po)
   priv->dl_branch = NULL;
   priv->dl_domain = NULL;
   priv->dl_state = NULL;
+  priv->dl_vcs_web = NULL;
 }
 
 static void
@@ -252,6 +238,10 @@ gtr_po_finalize (GObject * object)
     g_free (priv->dl_domain);
   if (priv->dl_state)
     g_free (priv->dl_state);
+  g_clear_pointer (&priv->dl_vcs_web, g_free);
+
+  if (priv->iter)
+    po_message_iterator_free (priv->iter);
 
   G_OBJECT_CLASS (gtr_po_parent_class)->finalize (object);
 }
@@ -262,6 +252,7 @@ gtr_po_dispose (GObject * object)
   GtrPo *po = GTR_PO (object);
   GtrPoPrivate *priv = gtr_po_get_instance_private (po);
 
+  g_clear_object (&priv->header);
   g_clear_object (&priv->location);
 
   G_OBJECT_CLASS (gtr_po_parent_class)->dispose (object);
@@ -447,7 +438,7 @@ _gtr_po_load (GtrPo * po, GFile * location, GError ** error)
   po_message_iterator_t iter;
   po_message_t message;
   const gchar *msgid;
-  gchar *filename;
+  g_autofree char *filename = NULL;
   GtrPoPrivate *priv = gtr_po_get_instance_private (po);
 
   /*
@@ -480,7 +471,6 @@ _gtr_po_load (GtrPo * po, GFile * location, GError ** error)
     }
 
   priv->gettext_po_file = po_file_read (filename, &handler);
-  g_free (filename);
 
   if (po_file_is_empty (priv->gettext_po_file))
     {
@@ -497,7 +487,6 @@ _gtr_po_load (GtrPo * po, GFile * location, GError ** error)
                    GTR_PO_ERROR_FILENAME,
                    _("Failed opening file “%s”: %s"),
                    filename, g_strerror (errno));
-      g_free (filename);
       return FALSE;
     }
 
@@ -520,6 +509,8 @@ _gtr_po_load (GtrPo * po, GFile * location, GError ** error)
       priv->header = gtr_header_new (iter, message);
     }
 
+  if (priv->iter)
+    po_message_iterator_free (priv->iter);
   priv->iter = iter;
 
   return TRUE;
@@ -531,11 +522,10 @@ _gtr_po_load_ensure_utf8 (GtrPo * po, GError ** error)
   g_autofree gchar *content = NULL;
   gboolean utf8_valid;
   gsize size;
-  g_autoptr (GError) err = NULL;
   GtrPoPrivate *priv = gtr_po_get_instance_private (po);
 
-  g_file_load_contents (priv->location, NULL, &content, &size, NULL, &err);
-  if (err != NULL)
+  g_file_load_contents (priv->location, NULL, &content, &size, NULL, error);
+  if (*error != NULL)
     return FALSE;
 
   utf8_valid = g_utf8_validate (content, size, NULL);
@@ -650,7 +640,7 @@ _gtr_po_load_ensure_utf8 (GtrPo * po, GError ** error)
  * Parses all things related to the #GtrPo and initializes all necessary
  * variables.
  **/
-gboolean
+static bool
 gtr_po_parse (GtrPo * po, GFile * location, GError ** error)
 {
   GtrMsg *msg;
@@ -676,10 +666,7 @@ gtr_po_parse (GtrPo * po, GFile * location, GError ** error)
   priv->location = g_file_dup (location);
 
   if (!_gtr_po_load_ensure_utf8 (po, error))
-    {
-      g_object_unref (po);
-      return FALSE;
-    }
+    return FALSE;
 
   /*
    * No need to return; this can be corrected by the user
@@ -701,7 +688,6 @@ gtr_po_parse (GtrPo * po, GFile * location, GError ** error)
                    GTR_PO_ERROR,
                    GTR_PO_ERROR_GETTEXT,
                    _("Gettext returned a null message domain list."));
-      g_object_unref (po);
       return FALSE;
     }
   while (domains[i])
@@ -748,7 +734,6 @@ gtr_po_parse (GtrPo * po, GFile * location, GError ** error)
                    GTR_PO_ERROR,
                    GTR_PO_ERROR_OTHER,
                    _("No messages obtained from parser."));
-      g_object_unref (po);
       return FALSE;
     }
 
@@ -766,6 +751,18 @@ gtr_po_parse (GtrPo * po, GFile * location, GError ** error)
   return TRUE;
 }
 
+GtrPo*
+gtr_po_new_from_file (GFile   *location,
+                      GError **error)
+{
+  g_autoptr(GtrPo) po = gtr_po_new ();
+
+  if (!gtr_po_parse (po, location, error))
+    return NULL;
+
+  return g_steal_pointer (&po);
+}
+
 /**
  * gtr_po_save_file:
  * @po: a #GtrPo
@@ -823,6 +820,7 @@ gtr_po_save_file (GtrPo * po, GError ** error)
         }
     }
 
+  po_message_iterator_free (iter);
 
   if (is_read_only (filename))
     {
@@ -962,15 +960,25 @@ gtr_po_set_state (GtrPo * po, GtrPoState state)
   g_object_notify (G_OBJECT (po), "state");
 }
 
-void gtr_po_set_dl_info (GtrPo * po, gchar * lang, gchar * module_name,
-                         gchar * branch, gchar * domain, gchar * module_state)
+void
+gtr_po_set_dl_info (GtrPo *po,
+                    const gchar *lang,
+                    const gchar *module_name,
+                    const gchar *branch,
+                    const gchar *domain,
+                    const gchar *module_state,
+                    const gchar *vcs_web)
 {
   GtrPoPrivate *priv = gtr_po_get_instance_private (po);
-  priv->dl_lang = g_strdup (lang);
-  priv->dl_module = g_strdup (module_name);
-  priv->dl_branch = g_strdup (branch);
-  priv->dl_domain = g_strdup (domain);
-  priv->dl_state = g_strdup (module_state);
+  g_set_str (&priv->dl_lang, lang);
+  g_set_str (&priv->dl_module, module_name);
+  g_set_str (&priv->dl_branch, branch);
+  g_set_str (&priv->dl_domain, domain);
+  g_set_str (&priv->dl_state, module_state);
+  g_set_str (&priv->dl_vcs_web, vcs_web);
+
+  if (vcs_web)
+    gtr_header_set_field (priv->header, "X-DL-VCS-Web", vcs_web);
 
   if (lang)
     gtr_header_set_field (priv->header, "X-DL-Lang", lang);
@@ -1416,6 +1424,13 @@ gtr_po_get_dl_module_state (GtrPo *po)
   return priv->dl_state;
 }
 
+const gchar *
+gtr_po_get_dl_vcs_web (GtrPo *po)
+{
+  GtrPoPrivate *priv = gtr_po_get_instance_private (po);
+  return priv->dl_vcs_web;
+}
+
 gboolean
 gtr_po_can_dl_upload (GtrPo *po)
 {
