@@ -16,8 +16,6 @@
  *
  */
 
-#define G_LOG_DOMAIN "gtr-search-bar"
-
 #include "config.h"
 
 #include <glib/gi18n.h>
@@ -386,7 +384,7 @@ gtr_search_bar_grab_focus (GtkWidget *widget)
 }
 
 static void
-gtr_search_bar_destroy (GObject *object)
+gtr_search_bar_dispose (GObject *object)
 {
   GtrSearchBar *self = (GtrSearchBar *)object;
 
@@ -430,22 +428,27 @@ add_actions (GtrSearchBar *self)
 
   action = g_property_action_new ("case-sensitive", self, "case-sensitive");
   g_action_map_add_action (G_ACTION_MAP (group), G_ACTION (action));
+  g_object_unref (action);
 
   action = g_property_action_new ("at-word-boundaries", self, "at-word-boundaries");
   g_action_map_add_action (G_ACTION_MAP (group), G_ACTION (action));
+  g_object_unref (action);
 
   action = g_property_action_new ("wrap-around", self, "wrap-around");
   g_action_map_add_action (G_ACTION_MAP (group), G_ACTION (action));
+  g_object_unref (action);
 
   action = g_property_action_new ("at-original-text", self, "at-original-text");
   g_action_map_add_action (G_ACTION_MAP (group), G_ACTION (action));
+  g_object_unref (action);
 
   action = g_property_action_new ("at-translated-text", self, "at-translated-text");
   g_action_map_add_action (G_ACTION_MAP (group), G_ACTION (action));
+  g_object_unref (action);
 
   gtk_widget_insert_action_group (GTK_WIDGET (self), "search-settings", G_ACTION_GROUP (group));
 
-  g_object_unref (action);
+  g_object_unref (group);
 }
 
 
@@ -543,7 +546,7 @@ gtr_search_bar_class_init (GtrSearchBarClass *klass)
 
   object_class->get_property = gtr_search_bar_get_property;
   object_class->set_property = gtr_search_bar_set_property;
-  object_class->dispose = gtr_search_bar_destroy;
+  object_class->dispose = gtr_search_bar_dispose;
 
   widget_class->grab_focus = gtr_search_bar_grab_focus;
 
