@@ -25,7 +25,6 @@
 
 #include "gtr-settings.h"
 #include "gtr-application.h"
-#include "gtr-debug.h"
 #include "gtr-view.h"
 #include "gtr-window.h"
 #include "gtr-dirs.h"
@@ -61,6 +60,7 @@ gtr_settings_dispose (GObject * object)
   GtrSettingsPrivate *priv = gtr_settings_get_instance_private (gs);
 
   g_clear_object (&priv->interface);
+  g_clear_object (&priv->files);
   g_clear_object (&priv->editor);
   g_clear_object (&priv->ui);
 
