@@ -37,13 +37,11 @@
 #include "gtr-context.h"
 #include "gtr-io-error-info-bar.h"
 #include "gtr-msg.h"
-#include "gtr-tab-activatable.h"
 #include "gtr-tab.h"
 #include "gtr-po.h"
 #include "gtr-settings.h"
 #include "gtr-view.h"
 #include "gtr-dirs.h"
-#include "gtr-debug.h"
 #include "gtr-window.h"
 #include "gtr-progress.h"
 #include "gtr-actions.h"
@@ -113,7 +111,6 @@ typedef struct
   guint blocking : 1;
 
   guint tab_realized : 1;
-  guint dispose_has_run : 1;
 
   /*Search Bar*/
   GtkSearchBar   *search_bar;
@@ -241,17 +238,20 @@ gtr_page_notify_child_revealed (GtrTab *tab,
 
 //----------------------------------------------------------------//
 
-static gboolean
-show_hide_revealer (GtkWidget *widget, GdkEvent *ev, GtrTab *tab)
+static void
+show_hide_revealer (GtkGesture    *gesture,
+                    int            n_click,
+                    double         x,
+                    double         y,
+                    GtrTab        *tab)
 {
   GtrTabPrivate *priv;
   GtkRevealer *rev;
 
+  gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_CLAIMED);
   priv = gtr_tab_get_instance_private (tab);
   rev = GTK_REVEALER (priv->progress_revealer);
   gtk_revealer_set_reveal_child (rev, !gtk_revealer_get_reveal_child (rev));
-
-  return TRUE;
 }
 
 static gboolean
@@ -435,6 +435,7 @@ gtr_message_translation_update (GtkTextBuffer * textbuffer, GtrTab * tab)
 
       /* Write back to PO file in memory */
       gtr_msg_set_msgstr_plural (msg, i, translation);
+      g_free (translation);
       return;
     }
 
@@ -824,10 +825,9 @@ gtr_tab_dispose (GObject * object)
 {
   GtrTabPrivate *priv;
 
-  priv = gtr_tab_get_instance_private (GTR_TAB (object));
+  g_debug ("Disposing tab");
 
-  if (!priv->dispose_has_run)
-    priv->dispose_has_run = TRUE;
+  priv = gtr_tab_get_instance_private (GTR_TAB (object));
 
   g_clear_object (&priv->po);
   g_clear_object (&priv->ui_settings);
@@ -1048,7 +1048,7 @@ gtr_tab_new (GtrPo * po,
 
   priv->window = GTK_WIDGET (window);
   /* FIXME: make the po a property */
-  priv->po = po;
+  g_set_object (&priv->po, po);
   g_object_set_data (G_OBJECT (po), GTR_TAB_KEY, tab);
 
   g_signal_connect (po, "notify::location",
@@ -1534,7 +1534,8 @@ gtr_tab_copy_original (GtrTab * tab)
   clipboard = gtk_widget_get_clipboard (GTK_WIDGET (tab));
   gdk_clipboard_set_text (clipboard, text);
 
-  gtr_window_add_toast_msg (GTR_WINDOW (priv->window), _("Copied to clipboard"));
+  gtr_window_add_toast_msg (GTR_WINDOW (priv->window),
+                            _("Copied original to clipboard"));
 }
 
 /**
@@ -1597,7 +1598,7 @@ _gtr_tab_finish_edition (GtrTab * tab)
 void
 gtr_tab_go_to_next (GtrTab * tab)
 {
-  GtrMsg *msg;
+  g_autoptr(GtrMsg) msg = NULL;
   GtrTabPrivate *priv;
 
   priv = gtr_tab_get_instance_private (tab);
@@ -1621,7 +1622,7 @@ gtr_tab_go_to_next (GtrTab * tab)
 void
 gtr_tab_go_to_prev (GtrTab * tab)
 {
-  GtrMsg *msg;
+  g_autoptr(GtrMsg) msg = NULL;
   GtrTabPrivate *priv;
 
   priv = gtr_tab_get_instance_private (tab);
@@ -1645,7 +1646,7 @@ gtr_tab_go_to_prev (GtrTab * tab)
 void
 gtr_tab_go_to_first (GtrTab * tab)
 {
-  GtrMsg *msg;
+  g_autoptr(GtrMsg) msg = NULL;
   GtrTabPrivate *priv;
 
   priv = gtr_tab_get_instance_private (tab);
@@ -1669,7 +1670,7 @@ gtr_tab_go_to_first (GtrTab * tab)
 void
 gtr_tab_go_to_last (GtrTab * tab)
 {
-  GtrMsg *msg;
+  g_autoptr(GtrMsg) msg = NULL;
   GtrTabPrivate *priv;
 
   priv = gtr_tab_get_instance_private (tab);
@@ -1695,7 +1696,7 @@ gtr_tab_go_to_last (GtrTab * tab)
 gboolean
 gtr_tab_go_to_next_fuzzy (GtrTab * tab)
 {
-  GtrMsg *msg;
+  g_autoptr(GtrMsg) msg = NULL;
   GtrTabPrivate *priv;
 
   priv = gtr_tab_get_instance_private (tab);
@@ -1726,7 +1727,7 @@ gtr_tab_go_to_next_fuzzy (GtrTab * tab)
 gboolean
 gtr_tab_go_to_prev_fuzzy (GtrTab * tab)
 {
-  GtrMsg *msg;
+  g_autoptr(GtrMsg) msg = NULL;
   GtrTabPrivate *priv;
 
   priv = gtr_tab_get_instance_private (tab);
@@ -1769,7 +1770,7 @@ message_is_fuzzy_or_untranslated (GtrMsg * msg)
 gboolean
 gtr_tab_go_to_next_untrans (GtrTab * tab)
 {
-  GtrMsg *msg;
+  g_autoptr(GtrMsg) msg = NULL;
   GtrTabPrivate *priv;
 
   priv = gtr_tab_get_instance_private (tab);
@@ -1800,7 +1801,7 @@ gtr_tab_go_to_next_untrans (GtrTab * tab)
 gboolean
 gtr_tab_go_to_prev_untrans (GtrTab * tab)
 {
-  GtrMsg *msg;
+  g_autoptr(GtrMsg) msg = NULL;
   GtrTabPrivate *priv;
 
   priv = gtr_tab_get_instance_private (tab);
@@ -1831,7 +1832,7 @@ gtr_tab_go_to_prev_untrans (GtrTab * tab)
 gboolean
 gtr_tab_go_to_next_fuzzy_or_untrans (GtrTab * tab)
 {
-  GtrMsg *msg;
+  g_autoptr(GtrMsg) msg = NULL;
   GtrTabPrivate *priv;
 
   priv = gtr_tab_get_instance_private (tab);
@@ -1862,7 +1863,7 @@ gtr_tab_go_to_next_fuzzy_or_untrans (GtrTab * tab)
 gboolean
 gtr_tab_go_to_prev_fuzzy_or_untrans (GtrTab * tab)
 {
-  GtrMsg *msg;
+  g_autoptr(GtrMsg) msg = NULL;
   GtrTabPrivate *priv;
 
   priv = gtr_tab_get_instance_private (tab);
