@@ -32,7 +32,6 @@
 #include "gtr-utils.h"
 #include "gtr-view.h"
 #include "gtr-application.h"
-#include "gtr-debug.h"
 
 #include <glib.h>
 #include <glib/gi18n.h>
@@ -71,10 +70,10 @@ notify_dark_cb (GtrView *view)
 static void
 gtr_view_init (GtrView * view)
 {
-  GtkSourceLanguageManager *lm;
+  g_autoptr (GtkSourceLanguageManager) lm = NULL;
   GtkSourceLanguage *lang;
-  GPtrArray *dirs;
-  gchar **langs = NULL;
+  g_autoptr(GStrvBuilder) builder = g_strv_builder_new ();
+  g_auto(GStrv) langs = NULL;
   const gchar *const *temp;
   gchar *ui_dir;
   GtrViewPrivate *priv;
@@ -99,24 +98,20 @@ gtr_view_init (GtrView * view)
   priv->ui_settings = g_settings_new ("org.gnome.gtranslator.preferences.ui");
 
   lm = gtk_source_language_manager_new ();
-  dirs = g_ptr_array_new ();
 
   for (temp = gtk_source_language_manager_get_search_path (lm);
        temp != NULL && *temp != NULL; ++temp)
-    g_ptr_array_add (dirs, g_strdup (*temp));
+    g_strv_builder_add (builder, *temp);
 
   ui_dir = g_build_filename (gtr_dirs_get_gtr_data_dir (), "ui", NULL);
-  g_ptr_array_add (dirs, ui_dir);
-  g_ptr_array_add (dirs, NULL);
-  langs = (gchar **) g_ptr_array_free (dirs, FALSE);
+  g_strv_builder_take (builder, ui_dir);
+  langs = g_strv_builder_end (builder);
 
   gtk_source_language_manager_set_search_path (lm, (const  char * const *) langs);
   lang = gtk_source_language_manager_get_language (lm, "gtranslator");
-  g_strfreev (langs);
 
   priv->buffer = GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
   gtk_source_buffer_set_language (priv->buffer, lang);
-  gtk_text_view_set_buffer (GTK_TEXT_VIEW (view), GTK_TEXT_BUFFER (priv->buffer));
   gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (view), GTK_WRAP_WORD);
 
   /* Set syntax highlight according to preferences */
@@ -260,8 +255,7 @@ gtr_view_enable_visible_whitespace (GtrView * view, gboolean enable)
   GtkSourceView *source;
   GtkSourceSpaceDrawer *drawer;
   GtkSourceBuffer *buffer;
-  GtkSourceLanguageManager *manager;
-  GtkSourceLanguage *lang;
+  g_autoptr (GtkSourceLanguageManager) manager = NULL;
   const gchar * const * deflangs = NULL;
   const gchar *langs[20] = {NULL};
   gint i = 0;
@@ -280,14 +274,12 @@ gtr_view_enable_visible_whitespace (GtrView * view, gboolean enable)
 
   manager = gtk_source_language_manager_new ();
   gtk_source_language_manager_set_search_path (manager, (const char * const *)langs);
-  lang = gtk_source_language_manager_get_language (manager, "gtranslator");
 
   source = GTK_SOURCE_VIEW (view);
   drawer = gtk_source_view_get_space_drawer (source);
   gtk_source_space_drawer_set_enable_matrix (drawer, TRUE);
   buffer = GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
   gtk_source_buffer_set_implicit_trailing_newline (buffer, FALSE);
-  gtk_source_buffer_set_language (buffer, lang);
 
   gtk_source_buffer_set_highlight_syntax (buffer, TRUE);
 
