@@ -26,7 +26,6 @@
 
 #include "gtr-actions.h"
 #include "gtr-application.h"
-#include "gtr-debug.h"
 #include "gtr-dirs.h"
 #include "gtr-header.h"
 #include "gtr-msg.h"
@@ -37,7 +36,6 @@
 #include "gtr-settings.h"
 #include "gtr-utils.h"
 #include "gtr-window.h"
-#include "gtr-window-activatable.h"
 #include "gtr-profile-manager.h"
 #include "gtr-greeter.h"
 
@@ -88,8 +86,6 @@ typedef struct
   GtrProfileManager *prof_manager;
 
   gboolean search_bar_shown;
-
-  guint dispose_has_run : 1;
 } GtrWindowPrivate;
 
 struct _GtrWindow
@@ -146,19 +142,25 @@ drag_data_received_cb (GtkDropTarget * drop_target,
 {
   GtrWindow * window = GTR_WINDOW (data);
   g_autoptr (GError) error = NULL;
+  g_autoptr (GtrPo) po = NULL;
+  GFile *location;
 
   if (!G_VALUE_HOLDS (value, G_TYPE_FILE))
     return FALSE;
 
-  gtr_window_remove_tab (window);
-  gtr_open (g_value_get_object (value), window, &error);
+  location = g_value_get_object (value);
+
+  po = gtr_po_new_from_file (location, &error);
   if (error != NULL)
     {
       AdwDialog *dialog = adw_alert_dialog_new (NULL, error->message);
       adw_alert_dialog_add_response (ADW_ALERT_DIALOG (dialog), "ok", _("OK"));
       adw_alert_dialog_set_default_response (ADW_ALERT_DIALOG (dialog), "ok");
       adw_dialog_present (ADW_DIALOG (dialog), GTK_WIDGET (window));
+      return FALSE;
     }
+  gtr_window_set_po (window, po);
+
   return TRUE;
 }
 
@@ -169,7 +171,7 @@ set_window_title (GtrWindow * window, gboolean with_path)
   GtrPoState state;
   GtrTab *active_tab;
   GFile *file;
-  g_autofree gchar *title;
+  g_autofree gchar *title = NULL;
 
   if (with_path)
     {
@@ -223,6 +225,7 @@ gtr_window_init (GtrWindow *window)
   //GtkTargetList *tl;
   GtrWindowPrivate *priv = gtr_window_get_instance_private(window);
   GAction *sort_action;
+  GAction *show_id_column_action;
 
   priv->search_bar_shown = FALSE;
   priv->state_settings = g_settings_new ("org.gnome.gtranslator.state.window");
@@ -266,6 +269,10 @@ gtr_window_init (GtrWindow *window)
   sort_action
     = g_settings_create_action (priv->ui_settings, GTR_SETTINGS_SORT_ORDER);
   g_action_map_add_action (G_ACTION_MAP (window), sort_action);
+
+  show_id_column_action
+      = g_settings_create_action (priv->ui_settings, GTR_SETTINGS_SHOW_ID_COLUMN);
+  g_action_map_add_action (G_ACTION_MAP (window), show_id_column_action);
 }
 
 static void
@@ -288,11 +295,7 @@ gtr_window_dispose (GObject * object)
   GtrWindow *window = GTR_WINDOW (object);
   GtrWindowPrivate *priv = gtr_window_get_instance_private(window);
 
-  if (!priv->dispose_has_run)
-    {
-      save_window_state (window);
-      priv->dispose_has_run = TRUE;
-    }
+  g_debug ("Dispose window");
 
   g_clear_object (&priv->state_settings);
   g_clear_object (&priv->ui_settings);
@@ -310,6 +313,19 @@ gtr_window_finalize (GObject * object)
   G_OBJECT_CLASS (gtr_window_parent_class)->finalize (object);
 }
 
+static gboolean
+gtr_window_close_request (GtkWindow *window)
+{
+  gtr_file_quit (GTR_WINDOW (window));
+
+  save_window_state (GTR_WINDOW (window));
+
+  // Window is removed in gtr_file_quit, after confirmation of save
+  // dialog, return TRUE will avoid the close of the window and let
+  // the gtr_file_quit handle it with the close confirmation dialog
+  return TRUE;
+}
+
 void
 gtr_window_save_current_tab (GtrWindow *self)
 {
@@ -357,10 +373,13 @@ static void
 gtr_window_class_init (GtrWindowClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GtkWindowClass *window_class = GTK_WINDOW_CLASS (klass);
 
   object_class->finalize = gtr_window_finalize;
   object_class->dispose = gtr_window_dispose;
 
+  window_class->close_request = gtr_window_close_request;
+
   gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (klass),
                                                "/org/gnome/translator/gtr-window.ui");
 
@@ -456,6 +475,67 @@ gtr_window_create_tab (GtrWindow * window, GtrPo * po)
   return tab;
 }
 
+void
+gtr_window_set_po (GtrWindow *window, GtrPo *po)
+{
+  GtrTab *tab;
+  GList *current;
+  GtrView *active_view;
+  GtrHeader *header;
+  g_autofree gchar *dl_vcs_web  = NULL;
+  g_autofree gchar *dl_lang  = NULL;
+  g_autofree gchar *dl_module  = NULL;
+  g_autofree gchar *dl_branch  = NULL;
+  g_autofree gchar *dl_domain  = NULL;
+  g_autofree gchar *dl_module_state  = NULL;
+
+  header = gtr_po_get_header (po);
+  dl_lang = gtr_header_get_dl_lang (header);
+  dl_module = gtr_header_get_dl_module (header);
+  dl_branch = gtr_header_get_dl_branch (header);
+  dl_domain = gtr_header_get_dl_domain (header);
+  dl_module_state = gtr_header_get_dl_state (header);
+  dl_vcs_web = gtr_header_get_dl_vcs_web (header);
+
+  /*
+   * Set Damned Lies info when a po file is opened locally
+   */
+  gtr_po_set_dl_info (po,
+                      dl_lang,
+                      dl_module,
+                      dl_branch,
+                      dl_domain,
+                      dl_module_state,
+                      dl_vcs_web);
+
+  /*
+   * Create a page to add to our list of open files
+   */
+  gtr_window_remove_tab (window);
+  tab = gtr_window_create_tab (window, po);
+
+  /*
+   * Activate the upload file icon if the po file is in the appropriate
+   * state as on the vertimus workflow
+   */
+  //active_notebook = gtr_window_get_notebook (window);
+  gtr_tab_enable_upload (tab, gtr_po_can_dl_upload (po));
+
+  /*
+   * Show the current message.
+   */
+  current = gtr_po_get_current_message (po);
+  gtr_tab_message_go_to (tab, current->data, FALSE, GTR_TAB_MOVE_NONE);
+
+  /*
+   * Grab the focus
+   */
+  active_view = gtr_tab_get_active_view (tab);
+  gtk_widget_grab_focus (GTK_WIDGET (active_view));
+
+  gtr_window_show_poeditor (window);
+}
+
 /**
  * gtr_window_get_active_tab:
  * @window: a #GtrWindow
@@ -796,3 +876,35 @@ gtr_window_add_toast_msg (GtrWindow *window,
   adw_toast_set_timeout (toast, 5);
   gtr_window_add_toast (window, toast);
 }
+
+static void
+on_launch (GObject *object, GAsyncResult *result, gpointer user_data)
+{
+  g_autoptr (GError) error = NULL;
+  gtk_uri_launcher_launch_finish (GTK_URI_LAUNCHER (object), result, &error);
+
+  if (error)
+    g_error ("Could not open uri: %s", error->message);
+}
+
+void
+gtr_window_open_file_in_browser (GtrWindow  *self,
+                                 const char *vcs_web,
+                                 const char *module,
+                                 const char *file,
+                                 const char *branch_name,
+                                 int         line_number)
+{
+  g_autoptr(GtkUriLauncher) launcher = NULL;
+  g_autofree char *module_endpoint = NULL;
+  g_autofree char *uri = NULL;
+  g_autofree char *file_with_line = NULL;
+
+  file_with_line = g_strdup_printf ("%s#L%d", file, line_number);
+  uri = g_build_path ("/", vcs_web, "/-/blob/", branch_name, file_with_line, NULL);
+
+  g_debug ("Opening %s", uri);
+
+  launcher = gtk_uri_launcher_new (uri);
+  gtk_uri_launcher_launch (launcher, GTK_WINDOW (self), NULL, on_launch, NULL);
+}
