@@ -39,6 +39,8 @@ G_DECLARE_FINAL_TYPE (GtrWindow, gtr_window, GTR, WINDOW, AdwApplicationWindow)
 
 GtrTab *gtr_window_create_tab (GtrWindow * window, GtrPo * po);
 
+void gtr_window_set_po (GtrWindow * window, GtrPo * po);
+
 GtrTab *gtr_window_get_active_tab (GtrWindow * window);
 
 GList *gtr_window_get_all_tabs (GtrWindow * window);
@@ -77,4 +79,11 @@ void gtr_window_show_greeter (GtrWindow *window);
 
 void gtr_window_save_current_tab (GtrWindow *window);
 
+void gtr_window_open_file_in_browser (GtrWindow  *self,
+                                      const char *vcs_web,
+                                      const char *module,
+                                      const char *file,
+                                      const char *branch_name,
+                                      int         line_number);
+
 G_END_DECLS
