@@ -14,9 +14,7 @@
  *     You should have received a copy of the GNU General Public License
  *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-
-#ifndef __GDA_BACKEND_H__
-#define __GDA_BACKEND_H__
+#pragma once
 
 #include <glib.h>
 #include <glib-object.h>
@@ -24,6 +22,8 @@
 
 G_BEGIN_DECLS
 
+#define GTR_GDA_ERROR (gtr_gda_error_quark ())
+
 #define GTR_TYPE_GDA		(gtr_gda_get_type ())
 
 G_DECLARE_FINAL_TYPE (GtrGda, gtr_gda, GTR, GDA, GObject)
@@ -33,12 +33,8 @@ struct _GtrGda
   GObject parent_instance;
 };
 
-struct _GtrGdaClass
-{
-  GObjectClass parent_class;
-};
+GtrGda                 *gtr_gda_new                     (void) G_GNUC_WARN_UNUSED_RESULT;
 
-GtrGda                 *gtr_gda_new                     (void);
+GQuark                  gtr_gda_error_quark             (void) G_GNUC_CONST;
 
 G_END_DECLS
-#endif /* __GDA_BACKEND_H__ */
