@@ -71,7 +71,7 @@ select_folder_cb (GtkFileDialog *dialog, GAsyncResult *res, gpointer user_data)
 static void
 select_directory_activated_cb (GtrTranslationMemoryDialog *dlg)
 {
-  GtkFileDialog *dialog;
+  g_autoptr(GtkFileDialog) dialog = NULL;
 
   dialog = gtk_file_dialog_new ();
   gtk_file_dialog_set_title (dialog, _("Select PO directory"));
@@ -153,20 +153,17 @@ add_to_database (gpointer data_pointer)
     {
       GList *msg_list = NULL;
       GFile *location;
-      GError *error = NULL;
-      GtrPo *po = NULL;
+      g_autoptr(GError) error = NULL;
+      g_autoptr(GtrPo) po = NULL;
 
-      po = gtr_po_new ();
       location = (GFile *) l->data;
-
-      gtr_po_parse (po, location, &error);
+      po = gtr_po_new_from_file (location, &error);
       if (error)
         return TRUE;
 
       msg_list = gtr_po_get_messages (po);
 
       gtr_translation_memory_store_list (data->tm, msg_list);
-      g_object_unref (po);
     }
   else
     {
@@ -217,6 +214,11 @@ task_data_destroy (ScanDirTaskData *data)
   g_free (data);
 }
 
+static void
+g_slist_free_files (GSList *list) {
+  g_slist_free_full (list, g_object_unref);
+}
+
 static void
 scan_dir_task_func (GTask                      *task,
                     GtrTranslationMemoryDialog *dlg,
@@ -225,7 +227,7 @@ scan_dir_task_func (GTask                      *task,
 {
   GSList *list = NULL;
   gtr_scan_dir (data->dir, &list, data->restriction);
-  g_task_return_pointer (task, list, (GDestroyNotify)g_slist_free);
+  g_task_return_pointer (task, list, (GDestroyNotify)g_slist_free_files);
 }
 
 static void
