@@ -29,7 +29,6 @@
 #include "gtr-tab.h"
 #include "gtr-utils.h"
 #include "gtr-window.h"
-#include "gtr-debug.h"
 
 #include <string.h>
 #include <glib.h>
@@ -166,13 +165,15 @@ showed_message_cb (GtrTab *tab, GtrMsg *msg, GtrTranslationMemoryUi *tm_ui)
   g_strfreev (priv->tm_list);
 
   priv->tm_list = g_new (gchar *, MAX_ELEMENTS + 1);
+  if (priv->tm_list_id)
+    g_free (priv->tm_list_id);
   priv->tm_list_id = g_new (gint, MAX_ELEMENTS + 1);
 
   i = 0;
   for (l = tm_list; l && i < MAX_ELEMENTS; l = l->next)
     {
       GtrTranslationMemoryMatch *match = (GtrTranslationMemoryMatch *) l->data;
-      GtrTm *tm = gtr_tm_new (GDK_KEY_1 + i, match->level, match->match);
+      g_autoptr(GtrTm) tm = gtr_tm_new (GDK_KEY_1 + i, match->level, match->match);
 
       priv->tm_list_id[i] = match->id;
       priv->tm_list[i] = g_strdup (match->match);
@@ -193,11 +194,11 @@ row_activated (GtkListView            *view,
                uint                    position,
                GtrTranslationMemoryUi *tm_ui)
 {
-  GtkSingleSelection *model;
-  GtrTm *tm;
+  GListModel *model;
+  g_autoptr(GtrTm) tm = NULL;
 
-  model = GTK_SINGLE_SELECTION (gtk_list_view_get_model (view));
-  tm = GTR_TM (g_list_model_get_object (gtk_single_selection_get_model (model), position));
+  model = G_LIST_MODEL (gtk_list_view_get_model (view));
+  tm = GTR_TM (g_list_model_get_object (model, position));
   choose_translation (tm_ui, tm->match);
 }
 
@@ -207,16 +208,14 @@ setup_widget (GtkSignalListItemFactory *factory,
 {
   GtkWidget *box, *child;
 
-  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 3);
-  gtk_widget_add_css_class (box, "message-table-row");
+  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+  gtk_widget_set_margin_top (box, 6);
+  gtk_widget_set_margin_bottom (box, 6);
   gtk_list_item_set_child (list_item, box);
 
   // Shortcut
-  // child = gtk_shortcut_label_new (NULL); // This widget is too big
-  child = gtk_label_new (NULL);
-  gtk_widget_set_vexpand (child, FALSE);
+  child = adw_shortcut_label_new ("");
   gtk_widget_set_valign (child, GTK_ALIGN_CENTER);
-  gtk_label_set_xalign (GTK_LABEL (child), 0.0);
   gtk_box_append (GTK_BOX (box), child);
 
   // Level
@@ -247,8 +246,8 @@ bind_widget (GtkSignalListItemFactory *factory,
 
   // shortcut
   child = gtk_widget_get_first_child (box);
-  sc = g_strdup_printf ("Ctrl+%s", gdk_keyval_name (tm->shortcut));
-  gtk_label_set_text (GTK_LABEL (child), sc);
+  sc = g_strdup_printf ("<Ctrl>%s", gdk_keyval_name (tm->shortcut));
+  adw_shortcut_label_set_accelerator (ADW_SHORTCUT_LABEL (child), sc);
 
   // level
   child = gtk_widget_get_next_sibling (child);
@@ -275,7 +274,7 @@ gtr_translation_memory_ui_init (GtrTranslationMemoryUi * tm_ui)
   g_signal_connect (factory, "setup", G_CALLBACK (setup_widget), NULL);
   g_signal_connect (factory, "bind", G_CALLBACK (bind_widget), NULL);
 
-  model = GTK_SELECTION_MODEL (gtk_single_selection_new ((G_LIST_MODEL (priv->model))));
+  model = GTK_SELECTION_MODEL (gtk_no_selection_new ((G_LIST_MODEL (priv->model))));
   priv->view = gtk_list_view_new (model, factory);
   gtk_widget_add_css_class (priv->view, "navigation-sidebar");
   gtk_widget_set_vexpand (priv->view, TRUE);
@@ -290,7 +289,7 @@ gtr_translation_memory_ui_dispose (GObject * object)
   GtrTranslationMemoryUi *tm_ui = GTR_TRANSLATION_MEMORY_UI (object);
   GtrTranslationMemoryUiPrivate *priv = gtr_translation_memory_ui_get_instance_private (tm_ui);
 
-  DEBUG_PRINT ("Dispose translation memory ui");
+  g_debug ("Dispose translation memory ui");
 
   g_clear_object (&priv->msg);
 
@@ -303,9 +302,10 @@ gtr_translation_memory_ui_finalize (GObject * object)
   GtrTranslationMemoryUi *tm_ui = GTR_TRANSLATION_MEMORY_UI (object);
   GtrTranslationMemoryUiPrivate *priv = gtr_translation_memory_ui_get_instance_private (tm_ui);
 
-  DEBUG_PRINT ("Finalize translation memory ui");
+  g_debug ("Finalize translation memory ui");
 
   g_strfreev (priv->tm_list);
+  g_free (priv->tm_list_id);
 
   G_OBJECT_CLASS (gtr_translation_memory_ui_parent_class)->finalize (object);
 }
